//Scan beacon + GPS.Vision 1

#include <stdio.h>
#include <stdint.h>
#include <stdbool.h>
#include "nordic_common.h"
#include "app_error.h"
#include "app_uart.h"
#include "app_trace.h"
#include "ble_db_discovery.h"
#include "app_timer.h"
#include "app_trace.h"
#include "app_util.h"
#include "app_error.h"
#include "bsp.h"
#include "bsp_btn_ble.h"
#include "boards.h"
#include "ble.h"
#include "ble_gap.h"
#include "ble_hci.h"
#include "softdevice_handler.h"
#include "ble_advdata.h"
#include "nrf_log.h"
#include "nrf_delay.h"
#include "mpu9250.h"


#define CENTRAL_LINK_COUNT      1                               /**< Number of central links used by the application. When changing this number remember to adjust the RAM settings*/
#define PERIPHERAL_LINK_COUNT   0                               /**< Number of peripheral links used by the application. When changing this number remember to adjust the RAM settings*/

#define UART_TX_BUF_SIZE        256                             /**< UART TX buffer size. */
#define UART_RX_BUF_SIZE        1                               /**< UART RX buffer size. */

#define NUS_SERVICE_UUID_TYPE   BLE_UUID_TYPE_VENDOR_BEGIN      /**< UUID type for the Nordic UART Service (vendor specific). */

#define APP_TIMER_PRESCALER     0                               /**< Value of the RTC1 PRESCALER register. */
#define APP_TIMER_OP_QUEUE_SIZE 2                               /**< Size of timer operation queues. */

#define APPL_LOG                app_trace_log                   /**< Debug logger macro that will be used in this file to do logging of debug information over UART. */

#define SCAN_INTERVAL           0x00A0                          /**< Determines scan interval in units of 0.625 millisecond. */
#define SCAN_WINDOW             0x0050                          /**< Determines scan window in units of 0.625 millisecond. */
#define SCAN_ACTIVE             0                               /**< If 1, performe active scanning (scan requests). */
#define SCAN_SELECTIVE          0                               /**< If 1, ignore unknown devices (non whitelisted). */
#define SCAN_TIMEOUT            0x0000                          /**< Timout when scanning. 0x0000 disables timeout. */

#define TIMEOUT_INTERVAL        APP_TIMER_TICKS(1000, APP_TIMER_PRESCALER)

#define UUID16_SIZE             2                               /**< Size of 16 bit UUID */
#define UUID32_SIZE             4                               /**< Size of 32 bit UUID */
#define UUID128_SIZE            16                              /**< Size of 128 bit UUID */

#define LORA_RX_PIN             11
#define LORA_TX_PIN             12

#define GPS_RX_PIN              5
#define GPS_TX_PIN              6

#define MPU_SCL_PIN             15
#define MPU_SDA_PIN             16

static const nrf_drv_twi_t m_twi_instance = NRF_DRV_TWI_INSTANCE(0);

static uint8_t ubeacon[16]= {0x54, 0x12, 0x48, 0x34, \
                             0x45, 0x81, 0x67, 0x12, \
                             0x02, 0x9a, 0x82, 0xbc, \
                             0xcd, 0x13, 0xef, 0x96};           /**Check beacon is me.*/

                             
static uint8_t p_data[20];                                      //beacon uuid
bool gps_check_data = false;                                    //if gps have GPGGA data is true,not data is false.                             
uint8_t gps_cnt = 0;                                            //Each 10 times, gps send.
char  gps_data[33];                                             //GPS sent data.In function gps_format()
uint8_t gps_no_data[32] = "AT+DTX=22,470FFFFFFFFFFFFFFFFFFF";
uint8_t array_len;
static uint8_t check_outdoor = 0;                                  //outdoor =1; indoor =0.
static int8_t  ble_rssi;                                         //RSSI Data.
static uint8_t lora_atz[3] = "ATZ";
static char beacon_data[33];          
                             
APP_TIMER_DEF(m_timeout_timer_id);

                             
/**
 * @brief Parameters used when scanning.
 */
static const ble_gap_scan_params_t m_scan_params = {
    .active      = SCAN_ACTIVE,
    .selective   = SCAN_SELECTIVE,
    .p_whitelist = NULL,
    .interval    = SCAN_INTERVAL,
    .window      = SCAN_WINDOW,
    .timeout     = SCAN_TIMEOUT
};
 
  
/**@brief Function for asserts in the SoftDevice.
 *
 * @details This function will be called in case of an assert in the SoftDevice.
 *
 * @warning This handler is an example only and does not fit a final product. You need to analyze
 *          how your product is supposed to react in case of Assert.
 * @warning On assert from the SoftDevice, the system can only recover on reset.
 *
 * @param[in] line_num     Line number of the failing ASSERT call.
 * @param[in] p_file_name  File name of the failing ASSERT call.
 */
void assert_nrf_callback(uint16_t line_num, const uint8_t * p_file_name)
{
    app_error_handler(0xDEADBEEF, line_num, p_file_name);
}


/**<*************************************************GPS*********************************************************/
static void pattern(uint8_t *info)
{
	uint8_t GPS_FullInfo[100];
	uint8_t index = 0;                                            //GPS_FullInfo index.
  uint8_t gps_comma_cnt = 1;                                    //GPS_FullInfo "," count.
  uint8_t gps_check_comma;                                      //GPS_FullInfo "," index.
  bool    gps_rx_done = 1;                                      //1 is no read gps. 0 is read.
  uint8_t lati_comma_pre;                                       //comma previous
  uint8_t lati_comma_af;                                        //comma after
  uint8_t longi_comma_pre;                                      //comma previous
  uint8_t longi_comma_af;                                       //comma after
  
  char longi_array[10];
  char lati_array[9];
  uint8_t j=0,k=0;
  uint8_t data_cnt;
  
  *GPS_FullInfo = *info;
  
	while(gps_rx_done)
  {
      uint8_t cr;
      while(app_uart_get(&cr) != NRF_SUCCESS);
      
      if(cr == '\n')
      {
          //if GPS_FullInfo[] = $GPGGA
          if(GPS_FullInfo[3] == 'R' && GPS_FullInfo[4] == 'M')
          {
              //NRF_LOG_PRINTF("GPS data is :\n\n%s\n\n", GPS_FullInfo);
              //NRF_LOG_PRINTF("gps_check_comma is %c-%d\n\nLati_comma is %c-%d\n\nLongi_comma is %c-%d\n\n", GPS_FullInfo[gps_check_comma+1], gps_check_comma, GPS_FullInfo[lati_comma_pre], lati_comma_pre, GPS_FullInfo[lati_comma_af], lati_comma_af, GPS_FullInfo[longi_comma_pre], longi_comma_pre, GPS_FullInfo[longi_comma_af], longi_comma_af);
              //NRF_LOG_PRINTF("GPS_CHECK is %c\n\n", GPS_FullInfo[gps_check_comma+1]);
              //if GPS_FullInfo[12] != 0.formate gps data
              if(GPS_FullInfo[gps_check_comma+1] == 'A')
              {
                  //NRF_LOG_PRINTF("%s\n\n", GPS_FullInfo);
                  
                  for(uint8_t i=0; i<10; i++)
                  {
                      lati_array[i-j] = GPS_FullInfo[lati_comma_pre+i+1];
                      if(lati_array[i] == '.') j++;
                      if(GPS_FullInfo[lati_comma_pre+i+1] == GPS_FullInfo[lati_comma_af])
                      {
                          lati_array[i-j] = '\0';
                          i = 100;
                      }
                      //NRF_LOG_PRINTF("%c", lati_array[i-j]);
                  }
                  
                  //NRF_LOG_PRINTF("Lati array is %s\n\n", lati_array);
                
                  for(uint8_t i=0; i<11; i++)
                  {
                      longi_array[i-k] = GPS_FullInfo[longi_comma_pre+i+1];
                      if(longi_array[i] == '.') k++;
                      if(GPS_FullInfo[longi_comma_pre+i+1] == GPS_FullInfo[longi_comma_af])
                      {
                          longi_array[i-j] = '\0';
                          i = 100;
                      }
                  }
                  
                  //NRF_LOG_PRINTF("Longi array is %s\n\n", longi_array);
                  if(lati_comma_af-lati_comma_pre == 11)
                  {
                      data_cnt = 22;
                      array_len = 32;
                  }else
                  {
                      data_cnt = 20;
                      array_len = 30;
                  }
                                    
                  snprintf( gps_data, 33, "AT+DTX=%d,48%d%s%s", data_cnt, lati_comma_af-lati_comma_pre-6, lati_array, longi_array);
                  NRF_LOG_PRINTF("gps_data is\n\n%s\n\n", gps_data);
                  gps_check_data = true;
                
              }else
              {
                  NRF_LOG_PRINTF("NO DATA...GPS status is %c\n\n", GPS_FullInfo[gps_check_comma+1]);
                  gps_check_data = false;
              }
              
              gps_rx_done = 0;
          }
          
          for(uint8_t i=0; i < sizeof(GPS_FullInfo); i++)
          {
              GPS_FullInfo[i] = '\0';
          }
          index = 0;
          gps_comma_cnt = 1;
      }else
      {      
          GPS_FullInfo[index] = cr;
          
        
          
          switch(gps_comma_cnt)
          {
            case 2:
              gps_check_comma = index;
              
              break;
            
            case 3:
              lati_comma_pre = index;
              break;
            
            case 4:
              lati_comma_af = index;
              break;
            
            case 5:
              longi_comma_pre = index;
              break;
            
            case 6:
              longi_comma_af = index;
              break;
            
            default:
              break;
          }
          
          if(cr == ',') gps_comma_cnt ++;
          //if(gps_check_cnt == 6) gps_check_comma = index;
          index++;
      }
  }
}

static void pattern1(uint8_t *info)
{
    static uint8_t GPS_FullInfo[100];
    uint8_t gps_comma_cnt=0, gps_check_comma,
            lati_comma_pre, lati_comma_af,
            longi_comma_pre, longi_comma_af;
    uint8_t lati_array[9], longi_array[10];
    uint8_t j=0, k=0;
    uint8_t data_cnt;

    *GPS_FullInfo = *info;

    for(uint8_t i=0; i<100; i++)
    {
      if(GPS_FullInfo[i] == '\n') break;
      if(GPS_FullInfo[i] == ',') gps_comma_cnt++;

      switch(gps_comma_cnt)
      {
        case 2:
          gps_check_comma = i;
          break;

        case 3:
          lati_comma_pre = i;
          break;

        case 4:
          lati_comma_af = i;
          break;

        case 5:
          longi_comma_pre = i;
          break;

        case 6:
          longi_comma_af = i;
          break;

        default:
          break;
      }

      if(GPS_FullInfo[3]=='R' && GPS_FullInfo[4]=='M')
      {
        if(GPS_FullInfo[gps_check_comma]=='A')
        {
          for(uint8_t i=0; i<lati_comma_af-lati_comma_pre; i++)
          {
            lati_array[i-j] = GPS_FullInfo[lati_comma_pre+i+1];
            if(lati_array[i-j]=='.') j++;
            if(GPS_FullInfo[lati_comma_pre+i+1]==GPS_FullInfo[lati_comma_af])
            {
              lati_array[i-j] = '\0';
              i = 100;
            }
          }
          for(uint8_t i=0; i<longi_comma_af-longi_comma_pre; i++)
          {
            longi_array[i-k] = GPS_FullInfo[longi_comma_pre+i+1];
            if(longi_array[i-k]=='.') k++;
            if(GPS_FullInfo[longi_comma_pre+i+1]==GPS_FullInfo[longi_comma_af])
            {
              longi_array[i-k] = '\0';
              i = 100;
            }
          }

                  if(lati_comma_af-lati_comma_pre == 11)
                  {
                      data_cnt = 22;
                      array_len = 32;
                  }else
                  {
                      data_cnt = 20;
                      array_len = 30;
                  }

                  snprintf( gps_data, 33, "AT+DTX=%d,48%d%s%s", data_cnt, lati_comma_af-lati_comma_pre-6, lati_array, longi_array);
                  NRF_LOG_PRINTF("gps_data is\n\n%s\n\n", gps_data);
        }else NRF_LOG_PRINTF("NO DATA...GPS status is %c\n\n", GPS_FullInfo[gps_check_comma+1]);
      }
    }
}

/**@brief Function to start scanning.
 */
static void scan_start(void)
{
    uint32_t err_code;
    
    err_code = sd_ble_gap_scan_start(&m_scan_params);
    APP_ERROR_CHECK(err_code);
  
    err_code = bsp_indication_set(BSP_INDICATE_SCANNING);
    APP_ERROR_CHECK(err_code);
}


static bool check_beacon()
{
    for(uint8_t i=0; i<16; i++)
    {
        if(p_data[i] != ubeacon[i]) return false;
    } 
    return true;
}

/**@brief Function for handling the Application's BLE Stack events.
 *
 * @param[in] p_ble_evt  Bluetooth stack event.
 */
static void on_ble_evt(ble_evt_t * p_ble_evt)
{
    uint32_t              err_code;
    const ble_gap_evt_t * p_gap_evt = &p_ble_evt->evt.gap_evt;
    
    uint8_t bas = 0;
    uint32_t  index;
    

    switch (p_ble_evt->header.evt_id)
    {
        case BLE_GAP_EVT_ADV_REPORT:
        {
            const ble_gap_evt_adv_report_t * p_adv_report = &p_gap_evt->params.adv_report;

            for(index = 0; index < 20; index ++)
            {
                p_data[index] = p_adv_report->data[index+9];
            }
            
            if(check_beacon())
            {
                ble_rssi = p_adv_report->rssi;
                if(ble_rssi < 0)
                {
                    ble_rssi = ble_rssi^0xFF;
                    ble_rssi++;
                }
                check_outdoor = 0;
                NRF_LOG_PRINTF("AAA...\n\n");
                snprintf(beacon_data, 33, "AT+DTX=22,410%02x%02x%02x%02x0%03d%04d123",p_data[16], p_data[17], p_data[18], p_data[19], ble_rssi, bas);
                //NRF_LOG_PRINTF("%s\n\n", beacon_data);
            }
            break;
        }

        case BLE_GAP_EVT_TIMEOUT:
            if (p_gap_evt->params.timeout.src == BLE_GAP_TIMEOUT_SRC_SCAN)
            {
                scan_start();
            }
            else if (p_gap_evt->params.timeout.src == BLE_GAP_TIMEOUT_SRC_CONN)
            {

            }
            break;

        case BLE_GAP_EVT_SEC_PARAMS_REQUEST:
            // Pairing not supported
            err_code = sd_ble_gap_sec_params_reply(p_ble_evt->evt.gap_evt.conn_handle, BLE_GAP_SEC_STATUS_PAIRING_NOT_SUPP, NULL, NULL);
            APP_ERROR_CHECK(err_code);
            break;

        case BLE_GAP_EVT_CONN_PARAM_UPDATE_REQUEST:
            // Accepting parameters requested by peer.
            err_code = sd_ble_gap_conn_param_update(p_gap_evt->conn_handle,
                                                    &p_gap_evt->params.conn_param_update_request.conn_params);
            APP_ERROR_CHECK(err_code);
            break;
    
        default:
            break;
    }
}

/**@brief Function for dispatching a BLE stack event to all modules with a BLE stack event handler.
 *
 * @details This function is called from the scheduler in the main loop after a BLE stack event has
 *          been received.
 *
 * @param[in] p_ble_evt  Bluetooth stack event.
 */
static void ble_evt_dispatch(ble_evt_t * p_ble_evt)
{
    on_ble_evt(p_ble_evt);
    bsp_btn_ble_on_ble_evt(p_ble_evt);
}

/**@brief Function for initializing the BLE stack.
 *
 * @details Initializes the SoftDevice and the BLE event interrupt.
 */
static void ble_stack_init(void)
{
    uint32_t err_code;
    
    nrf_clock_lf_cfg_t clock_lf_cfg = NRF_CLOCK_LFCLKSRC;
    
    // Initialize the SoftDevice handler module.
    SOFTDEVICE_HANDLER_INIT(&clock_lf_cfg, NULL);
    
    ble_enable_params_t ble_enable_params;
    err_code = softdevice_enable_get_default_config(CENTRAL_LINK_COUNT,
                                                    PERIPHERAL_LINK_COUNT,
                                                    &ble_enable_params);
    APP_ERROR_CHECK(err_code);
    
    //Check the ram settings against the used number of links
    CHECK_RAM_START_ADDR(CENTRAL_LINK_COUNT,PERIPHERAL_LINK_COUNT);
    
    // Enable BLE stack.
    err_code = softdevice_enable(&ble_enable_params);
    APP_ERROR_CHECK(err_code);

    // Register with the SoftDevice handler module for BLE events.
    err_code = softdevice_ble_evt_handler_set(ble_evt_dispatch);
    APP_ERROR_CHECK(err_code);
}



/**< ********************************************************Handler**************************************************************/

/**@brief   Function for handling app_uart events.
 *
 * @details This function will receive a single character from the app_uart module and append it to 
 *          a string. The string will be be sent over BLE when the last character received was a 
 *          'new line' i.e '\n' (hex 0x0D) or if the string has reached a length of 
 *          @ref NUS_MAX_DATA_LENGTH.
 */
/**@snippet [Handling the data received over UART] */
void uart_event_handle(app_uart_evt_t * p_event)
{
    static    uint8_t data_array[100];
    static    uint8_t index = 0;
    uint32_t          err_code;

    switch (p_event->evt_type)
    {
        case APP_UART_DATA_READY:
            
           //NRF_LOG_PRINTF("123\n\n");
           err_code = app_uart_get(&data_array[index]);
           index++;
           /* TODO: Substitute '\n' in the statement below with the Modbus escape character, if 
               applicable */
           if ((data_array[index - 1] == '\n') || (index >= (100)))
           {
                //uart_callback(&data_array[0], index);
                //NRF_LOG_PRINTF("AAA : %s\n\n", data_array);
                pattern1(data_array);
                if (err_code != NRF_ERROR_INVALID_STATE)
                {
                    APP_ERROR_CHECK(err_code);
                }

                index = 0;
            }
        
            break;

        case APP_UART_COMMUNICATION_ERROR:
            APP_ERROR_HANDLER(p_event->data.error_communication);
            break;

        case APP_UART_FIFO_ERROR:
            APP_ERROR_HANDLER(p_event->data.error_code);
            break;
        
        case APP_UART_TX_EMPTY:
            NRF_LOG_PRINTF("APP_UART_TX_EMPTY\n\n");

        default:
            break;
    }
}

void twi_handler(nrf_drv_twi_evt_t const* p_event, void* p_context)
{
    mpu9250_twi_event_handler(p_event);
}

/**< ********************************************************Driver init**************************************************************/
/**@brief Function for initializing LORA in uart.
*/
static void uart_lora_init(void)
{
    uint32_t  err_code;
  
    const app_uart_comm_params_t comm_params =
    {
        .rx_pin_no    = LORA_RX_PIN,
        .tx_pin_no    = LORA_TX_PIN,
        .rts_pin_no   = false,
        .cts_pin_no   = false,
        .flow_control = APP_UART_FLOW_CONTROL_DISABLED,
        .use_parity   = false,
        .baud_rate    = UART_BAUDRATE_BAUDRATE_Baud9600
    };
    
    APP_UART_FIFO_INIT(&comm_params,
                        UART_RX_BUF_SIZE,
                        UART_TX_BUF_SIZE,
                        uart_event_handle,
                        APP_IRQ_PRIORITY_LOW,
                        err_code);
    APP_ERROR_CHECK(err_code);
}

/**@brief Function for initializing GPS in uart.
*/
static void uart_gps_init(void)
{
    uint32_t  err_code;
  
    const app_uart_comm_params_t gps_comm_params =
    {
        .rx_pin_no    = GPS_RX_PIN,
        .tx_pin_no    = GPS_TX_PIN,
        .rts_pin_no   = false,
        .cts_pin_no   = false,
        .flow_control = APP_UART_FLOW_CONTROL_DISABLED,
        .use_parity   = false,
        .baud_rate    = UART_BAUDRATE_BAUDRATE_Baud9600
    };
    
    APP_UART_FIFO_INIT(&gps_comm_params,
                        UART_RX_BUF_SIZE,
                        UART_TX_BUF_SIZE,
                        uart_event_handle,
                        APP_IRQ_PRIORITY_LOW,
                        err_code);
    APP_ERROR_CHECK(err_code);
}


static void uart_close(void)
{
    app_uart_close();
}

static void twi_mpu_init(void)
{
    uint32_t err_code;
  
    const nrf_drv_twi_config_t twi_mpu_config = {
        .scl                = MPU_SCL_PIN,
        .sda                = MPU_SDA_PIN,
        .frequency          = NRF_TWI_FREQ_400K,
        .interrupt_priority = APP_IRQ_PRIORITY_HIGH
    };
    
    err_code = nrf_drv_twi_init(&m_twi_instance, &twi_mpu_config, twi_handler, NULL);
    APP_ERROR_CHECK(err_code);
    
    nrf_drv_twi_enable(&m_twi_instance);
}

static void mpu_init(void)
{
    uint32_t err_code;
  
    err_code = mpu9250_init(&m_twi_instance);
    APP_ERROR_CHECK(err_code);
  
    mpu9250_config_t p_mpu_config = MPU9250_DEFAULT_CONFIG(); // Load default values
    p_mpu_config.smplrt_div = 19;   // Change sampelrate. Sample Rate = Gyroscope Output Rate / (1 + SMPLRT_DIV). 19 gives a sample rate of 50Hz
    p_mpu_config.accel_config.afs_sel = AFS_2G; // Set accelerometer full scale range to 2G
    err_code = mpu9250_config(&p_mpu_config); // Configure the MPU with above values
    APP_ERROR_CHECK(err_code); // Check for errors in return value 
}

/**< ********************************************************Application Area**************************************************************/

static void sendGPS_data(void)
{

    uart_gps_init();
    NRF_LOG_PRINTF("GPS...\n\n");
                      
    uart_close();
                        
    /**LORA send GPS data*/
    uart_lora_init();
              
    for(uint8_t i = 0; i < 3; i ++)
    {
        app_uart_put(lora_atz[i]);
        NRF_LOG_PRINTF("%c", lora_atz[i]);
    }
    app_uart_put(0x0D);
    NRF_LOG_PRINTF("\n\n");
                    
    nrf_delay_ms(1000);
                        
    if(gps_check_data)
    {
        for(uint8_t i = 0; i < 32; i ++)
        {
            app_uart_put(gps_data[i]);
            NRF_LOG_PRINTF("%c", gps_data[i]);
        }
    }
    else
    {
        for(uint8_t i = 0; i < 32; i ++)
        {
            app_uart_put(gps_no_data[i]);
            NRF_LOG_PRINTF("%c", gps_no_data[i]);
        }
    }
                        
    app_uart_put(0x0D);
    NRF_LOG_PRINTF("\n\n");
                        
    nrf_delay_ms(1000);
    uart_close();
}

static void sendMPU_data(void)
{
    static char        mpu_data[29];
    static int32_t     temp;
  
    uint32_t err_code;
  
    accel_values_t acc_value;
    mpu9250_read_accel(&acc_value);
    //NRF_LOG_PRINTF("X : %d\nY : %d\nZ : %d\n", acc_value.x, acc_value.y, acc_value.z);
                
    err_code = sd_temp_get(&temp);
    APP_ERROR_CHECK(err_code);
    //NRF_LOG_PRINTF("TEMP is %d\n\n", temp/4);
                
    snprintf(mpu_data, 29, "AT+DTX=18,434%04x%04x%04x%03d", acc_value.x, acc_value.y, acc_value.z, temp/4);
                
    uart_lora_init();
    nrf_delay_ms(1000);
              
    for(uint8_t i = 0; i < 3; i ++)
    {
        app_uart_put(lora_atz[i]);
        NRF_LOG_PRINTF("%c", lora_atz[i]);
    }
    app_uart_put(0x0D);
    NRF_LOG_PRINTF("\n\n");
                    
    nrf_delay_ms(1000);
                    
    for(uint8_t i = 0; i < 28; i ++)
    {
        app_uart_put(mpu_data[i]);
        NRF_LOG_PRINTF("%c", mpu_data[i]);
    }
                    
    app_uart_put(0x0D);
    NRF_LOG_PRINTF("\n\n");
                        
    nrf_delay_ms(1000);
    uart_close();
}

static void sendBeacon_data(void)
{
    uint32_t err_code;
    uint8_t aaa[10] = "asdasdasda";
  
    uart_lora_init();
    nrf_delay_ms(1000);
   /*           
    for(uint8_t i = 0; i < 3; i ++)
    {
        err_code = app_uart_put(lora_atz[i]);
        NRF_LOG_PRINTF("ERROR is %d\n\n",err_code);
        NRF_LOG_PRINTF("%c", lora_atz[i]);
    }
   */ 
    for(uint8_t i = 0; i < 10; i ++)
    {
        err_code = app_uart_put(aaa[i]);
        NRF_LOG_PRINTF("ERROR is %d\n\n",err_code);
        NRF_LOG_PRINTF("%c", aaa[i]);
    }
    app_uart_put(0x0D);
    NRF_LOG_PRINTF("\n\n");
        
    nrf_delay_ms(1000);
   /*     
    for(uint8_t i = 0; i < 32; i ++)
    {
        app_uart_put(beacon_data[i]);
        NRF_LOG_PRINTF("%c", beacon_data[i]);
    }
        
    app_uart_put(0x0D);
    NRF_LOG_PRINTF("\n\n");
            
    nrf_delay_ms(1000);
   */ 
    uart_close();
}

/**< ********************************************************TIMER AREA DOWN**************************************************************/
static void timeout_timeout_handler(void* p_context)
{
  /*
    if(check_outdoor >0)                                               //Outdoor send GPS,IMU data;Indoor send Beacon,GPS and IMU data.
    {
        sendGPS_data();
        sendMPU_data();
    }else
    {
        check_outdoor ++;
        sendBeacon_data();
        sendGPS_data();
        sendMPU_data();
    }
  */
    sendBeacon_data();
  
    
}

static void timer_init(void)
{
    uint32_t err_code;
  
    APP_TIMER_INIT(APP_TIMER_PRESCALER, APP_TIMER_OP_QUEUE_SIZE, NULL);
  
    err_code = app_timer_create(&m_timeout_timer_id,
                                 APP_TIMER_MODE_SINGLE_SHOT,
                                 timeout_timeout_handler);
    APP_ERROR_CHECK(err_code);
}

static void timer_start(void) 
{
    uint32_t err_code;
  
    err_code = app_timer_start(m_timeout_timer_id,
                               TIMEOUT_INTERVAL,
                               NULL);
  
    APP_ERROR_CHECK(err_code);
}
/**< ********************************************************TIMER AREA UP**************************************************************/


/** @brief Function for the Power manager.
 */
static void power_manage(void)
{
    uint32_t err_code = sd_app_evt_wait();

    APP_ERROR_CHECK(err_code);
}

int main(void)
{  
    timer_init();
    //twi_mpu_init();
    //mpu_init();
   
    ble_stack_init();

    // Start scanning for peripherals and initiate connection
    // with devices that advertise NUS UUID.
    timer_start();
    scan_start();
    NRF_LOG_PRINTF("Scan started\r\n");

    for (;;)
    {
        power_manage();
        
    }
}
